SystemOrganization addCategory: #'Pier-Pharo-Persistency'!

!ImageSegment methodsFor: '*pier-pharo-persistency' stamp: 'lr 3/26/2009 23:36'!
writeKernel: aKernel on: aStream
	"Serialize aKernel on aStream so that it can be loaded into another image later. Answer a list of outpointers, these are objects that are referenced from somewhere else."

	| previousMarker |
	self
		copyFromRoots: (Array with: aKernel)
		sizeHint: 1024 *1024
		areUnique: true.
	state = #activeCopy
		ifFalse: [ self error: 'The image segment has an invalid state.' ].
	previousMarker := endMarker.
	endMarker := nil.
	aStream fileOutClass: nil andObject: self.
	endMarker := previousMarker.
	^ outPointers copy! !

PRObject subclass: #PRExporterImporter
	instanceVariableNames: 'owner writeStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Pharo-Persistency'!

!PRExporterImporter methodsFor: 'accessing-configuration' stamp: 'lr 10/13/2010 12:42'!
emptyWriteStream
	self subclassResponsibility! !

!PRExporterImporter methodsFor: 'actions' stamp: 'lr 10/13/2010 12:44'!
export: aKernel on: aRequestContext
	aKernel exportOn: self.
	aRequestContext response
		document: self writeStream contents
			mimeType: self mimeType
			fileName: aKernel name , self extension;
		respond! !

!PRExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:24'!
extension
	self subclassResponsibility! !

!PRExporterImporter methodsFor: 'actions' stamp: 'lr 10/13/2010 13:07'!
import: aCollection into: aKernel
	| kernel stream |
	aCollection isNil
		ifTrue: [ ^ owner inform: 'Please specify an import.' ].
	[ stream := self readStreamOn: aCollection.
	kernel := stream next ]
		on: Error
		do: [ :err | ^ owner inform: 'Unable to import: ' , err messageText printString , '.' ].
	(kernel isKindOf: PRKernel)
		ifFalse: [ ^ owner inform: 'Invalid file format.' ].
	self replace
		ifTrue: [ 
			aKernel becomeForward: kernel.
			aKernel persistency desanitize ]
		ifFalse: [
			kernel root name: (PRAddCommand new
				uniqueName: kernel root name
				in: aKernel root).
			aKernel root addChild: kernel root ].
	self sanitizeImport: kernel root.
	^ kernel
! !

!PRExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:24'!
mimeType
	self subclassResponsibility! !

!PRExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:24'!
owner
	^ owner! !

!PRExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:24'!
owner: anObject
	owner := anObject! !

!PRExporterImporter methodsFor: 'accessing-configuration' stamp: 'lr 10/13/2010 12:47'!
readStreamOn: aCollection
	self subclassResponsibility! !

!PRExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:24'!
replace
	^ owner replace! !

!PRExporterImporter methodsFor: 'private' stamp: 'jmck 10/9/2010 15:46'!
sanitizeExport: aStructure
	"Some older versions of Pier have block contexts in their conditions, get rid of these."
	
	aStructure enumerator with all do: [ :structure |
		Smalltalk at: #PRComponent ifPresent: [ :class |
			(structure isKindOf: class) ifTrue: [
				structure settings keys do: [ :desc |
					desc conditions: (desc conditions
						reject: [ :cond | cond key isBlock ]) ] ] ] ]! !

!PRExporterImporter methodsFor: 'private' stamp: 'lr 10/13/2010 12:20'!
sanitizeImport: aStructure
	"After importing drop all the security settings as these users might not exist in the new kernel. Also refresh all the links so that they are properly printed when editing these pages."
	
	aStructure enumerator with all do: [ :structure |
		self replace ifFalse: [
			Smalltalk at: #PUSecurity ifPresent: [ :class |
				structure 
					removeDecoration: class
					ifAbsent: [ ] ] ].
		structure outgoingReferences do: [ :link |
			(link isInternal and: [ link isBroken not ])
				ifTrue: [ link update ] ] ]! !

!PRExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:42'!
writeStream
	^ writeStream ifNil: [ writeStream := self emptyWriteStream ]! !

PRExporterImporter subclass: #PRReferenceStreamExporterImporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Pharo-Persistency'!

!PRReferenceStreamExporterImporter class methodsFor: 'testing' stamp: 'lr 10/13/2010 12:26'!
isAbstract
	^ false! !

!PRReferenceStreamExporterImporter class methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:04'!
label
	^ 'Reference Stream'! !

!PRReferenceStreamExporterImporter methodsFor: 'accessing-configuration' stamp: 'lr 10/13/2010 12:42'!
emptyWriteStream
	^ ReferenceStream on: (RWBinaryOrTextStream on: ByteArray new)! !

!PRReferenceStreamExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:26'!
extension
	^ '.obj'! !

!PRReferenceStreamExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:26'!
mimeType
	^ 'application/binary'! !

!PRReferenceStreamExporterImporter methodsFor: 'accessing-configuration' stamp: 'lr 10/13/2010 12:56'!
readStreamOn: aCollection
	^ ReferenceStream on: (RWBinaryOrTextStream with: aCollection asByteArray) reset! !

PRExporterImporter subclass: #PRSixxExporterImporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Pharo-Persistency'!

!PRSixxExporterImporter class methodsFor: 'testing' stamp: 'lr 10/13/2010 12:48'!
isAbstract
	^ self sixxReadStreamClass isNil or: [ self sixxWriteStreamClass isNil ]! !

!PRSixxExporterImporter class methodsFor: 'accessing' stamp: 'lr 10/13/2010 13:03'!
label
	^ 'SIXX Serializer'! !

!PRSixxExporterImporter class methodsFor: 'private' stamp: 'lr 10/13/2010 12:46'!
sixxReadStreamClass
	^ Smalltalk classNamed: #SixxReadStream! !

!PRSixxExporterImporter class methodsFor: 'private' stamp: 'lr 10/13/2010 12:46'!
sixxWriteStreamClass
	^ Smalltalk classNamed: #SixxWriteStream! !

!PRSixxExporterImporter methodsFor: 'accessing-configuration' stamp: 'lr 10/13/2010 12:47'!
emptyWriteStream
	^ self class sixxWriteStreamClass on: (RWBinaryOrTextStream on: ByteString new)! !

!PRSixxExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:47'!
extension
	^ '.sxx'! !

!PRSixxExporterImporter methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:47'!
mimeType
	^ 'text/xml'! !

!PRSixxExporterImporter methodsFor: 'accessing-configuration' stamp: 'lr 10/13/2010 13:07'!
readStreamOn: aCollection
	^ self class sixxReadStreamClass on: aCollection greaseString! !

PRHistoryPersistency subclass: #PRDelayedPersistency
	instanceVariableNames: 'duration process message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Pharo-Persistency'!

!PRDelayedPersistency commentStamp: 'lr 2/1/2009 17:03' prior: 0!
A snapshot based persistency mechanism.!

!PRDelayedPersistency class methodsFor: 'descriptions' stamp: 'lr 1/29/2009 21:32'!
descriptionDuration
	^ MADurationDescription new
		comment: 'The minimum interval between two snapshots.';
		accessor: #duration;
		label: 'Duration';
		default: 1 hour;
		priority: 100;
		beRequired;
		yourself! !

!PRDelayedPersistency class methodsFor: 'descriptions' stamp: 'lr 1/29/2009 21:18'!
descriptionMessage
	^ MAStringDescription new
		accessor: #message;
		label: 'Message';
		group: 'Status';
		priority: 200;
		beReadonly;
		yourself! !

!PRDelayedPersistency class methodsFor: 'testing' stamp: 'lr 1/29/2009 21:15'!
isAbstract
	^ self name = #PRDelayedPersistency! !

!PRDelayedPersistency methodsFor: 'accessing' stamp: 'lr 1/29/2009 21:16'!
duration
	^ duration! !

!PRDelayedPersistency methodsFor: 'accessing' stamp: 'lr 1/29/2009 21:16'!
duration: aDuration
	"The minimum interval between two snapshots."

	duration := aDuration! !

!PRDelayedPersistency methodsFor: 'actions' stamp: 'lr 2/20/2009 00:29'!
execute: aCommand
	self mutex critical: [
		self apply: aCommand.
		aCommand isLogged ifTrue: [
			self add: aCommand context.
			self log: aCommand context.
			self start ] ]! !

!PRDelayedPersistency methodsFor: 'testing' stamp: 'lr 1/29/2009 21:26'!
isScheduled
	^ process notNil and: [ process isTerminated not ]! !

!PRDelayedPersistency methodsFor: 'private' stamp: 'lr 1/29/2009 21:21'!
log: aContext
	| stream |
	stream := FileDirectory default
		fileNamed: 'transactions.txt'.
	[ self log: aContext on: stream setToEnd ]
		ensure: [ stream close ]! !

!PRDelayedPersistency methodsFor: 'private' stamp: 'lr 1/29/2009 21:21'!
log: aContext on: aStream
	aStream nextPutAll: ' == '; nextPutAll: aContext command class label; nextPutAll: ' == '; cr.
	aStream nextPutAll: ' -- context -- '; cr.
	aContext description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext
				readUsing: each)); cr ].
	aStream nextPutAll: ' -- command -- '; cr.
	aContext command description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext command
				readUsing: each)); cr ].
	aStream cr; cr! !

!PRDelayedPersistency methodsFor: 'accessing' stamp: 'lr 2/1/2009 16:52'!
message
	^ message! !

!PRDelayedPersistency methodsFor: 'accessing' stamp: 'lr 2/1/2009 16:52'!
message: aString
	message := aString! !

!PRDelayedPersistency methodsFor: 'private' stamp: 'lr 1/29/2009 22:01'!
process
	(Delay forDuration: self duration)
		wait.
	Processor activeProcess
		priority: Processor highestPriority.
	process := nil.
	self snapshot! !

!PRDelayedPersistency methodsFor: 'actions' stamp: 'lr 2/6/2009 22:33'!
sanitize
	super sanitize.
	self stop! !

!PRDelayedPersistency methodsFor: 'initialization' stamp: 'lr 1/29/2009 21:20'!
setKernel: aKernel
	super setKernel: aKernel.
	self stop! !

!PRDelayedPersistency methodsFor: 'actions' stamp: 'lr 1/29/2009 21:31'!
snapshot
	"Take a snapshot and perform cleanup tasks."

	self subclassResponsibility! !

!PRDelayedPersistency methodsFor: 'actions' stamp: 'lr 1/29/2009 21:30'!
start
	"Schedule the receiver to take a snapshot after the configured duration."

	self isScheduled
		ifTrue: [ ^ self ].
	process := [ self process ]
		newProcess.
	process 
		priority: Processor userBackgroundPriority;
		name: 'pier'.
	process resume! !

!PRDelayedPersistency methodsFor: 'actions' stamp: 'lr 2/20/2009 00:31'!
stop
	"Stop the configured snapshot."

	self isScheduled
		ifTrue: [ process terminate ].
	process := nil! !

PRDelayedPersistency subclass: #PRImagePersistency
	instanceVariableNames: 'namePrefix maxImages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Pharo-Persistency'!

!PRImagePersistency commentStamp: 'lr 2/1/2009 17:08' prior: 0!
A persistency strategy regularly saving the complete image. Do not use this strategy while developing, as this might corrupt the changes file.!

!PRImagePersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 1/23/2009 14:02'!
descriptionMaxImages
	^ MANumberDescription new
		comment: 'The maximum number of backup copies to keep';
		accessor: #maxImages;
		label: 'Backup Images';
		priority: 110;
		beRequired;
		default: 3;
		beInteger;
		yourself! !

!PRImagePersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 1/23/2009 14:03'!
descriptionNamePrefix
	^ MAStringDescription new
		comment: 'Defines the prefix for image-names.';
		default: 'pier_backup_';
		accessor: #namePrefix;
		label: 'Backup Prefix';
		priority: 120;
		beRequired;
		yourself! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 1/29/2009 21:31'!
cleanupBackupImages
	| images |
	maxImages isNil ifTrue: [ ^ self ].
	images := FileDirectory default fileNames select: [ :each | 
		(each endsWith: FileDirectory imageSuffix) 
			and: [ each beginsWith: namePrefix ] ].
	images size <= maxImages ifTrue: [ ^ self ].
	images := (images asSortedCollection allButLast: maxImages)
		collect: [ :each | each copyUpToLast: $. ].
	images do: [ :each |
		FileDirectory default 
			deleteFileNamed: (each , '.' , FileDirectory imageSuffix)
			ifAbsent: [ ].
		FileDirectory default 
			deleteFileNamed: (each , '.' , FileDirectory changeSuffix)
			ifAbsent: [ ] ]! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 17:54'!
maxImages
	^ maxImages! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 1/29/2009 21:27'!
maxImages: anInteger
	"The maximum number of backup copies to keep."

	maxImages := anInteger! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 18:06'!
namePrefix
	^ namePrefix! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 1/29/2009 21:27'!
namePrefix: aString
	"Defines the prefix for backup copies."

	namePrefix := aString! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 4/20/2009 22:46'!
saveImageAndBackupAs: aString
	| directory image imageName changeName |
	directory := FileDirectory default.
	image := SmalltalkImage current.
	image closeSourceFiles.
	imageName := aString , '.' , FileDirectory imageSuffix.
	changeName := aString , '.' , FileDirectory changeSuffix.
	((directory fileExists: imageName) not and: [ (directory fileExists: changeName) not ]) ifTrue: [
		directory
			copyFileWithoutOverwriteConfirmationNamed: image imageName 
			toFileNamed: (directory fullPathFor: imageName).
		directory
			copyFileWithoutOverwriteConfirmationNamed: image changesName 
			toFileNamed: (directory fullPathFor: changeName).	
		Smalltalk setMacFileInfoOn: (directory fullPathFor: changeName) ].
	image openSourceFiles; saveImageSegments; snapshot: true andQuit: false! !

!PRImagePersistency methodsFor: 'actions' stamp: 'jf 11/23/2009 20:58'!
snapshot
	| filename |
	filename := namePrefix , DateAndTime now asSortableString.
	[ self saveImageAndBackupAs: filename ]
		valueUnpreemptively.
	self message: 'Last snapshot ' , filename printString , ' at ' , TimeStamp now greaseString.
	self cleanupBackupImages! !

PRDelayedPersistency subclass: #PRSegmentPersistency
	instanceVariableNames: 'namePrefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Pharo-Persistency'!

!PRSegmentPersistency commentStamp: 'lr 2/1/2009 17:05' prior: 0!
A persistency strategy regularly saving an image segment of the kernel. This requires the OS process plugin to work.!

!PRSegmentPersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 2/5/2009 16:35'!
descriptionNamePrefix
	^ MAStringDescription new
		comment: 'Defines the prefix for snapshot files.';
		label: 'Snapshot Prefix';
		accessor: #namePrefix;
		default: 'snapshot_';
		priority: 120;
		beRequired;
		yourself! !

!PRSegmentPersistency class methodsFor: 'testing' stamp: 'lr 1/30/2009 16:55'!
isAbstract
	^ true! !

!PRSegmentPersistency methodsFor: 'private' stamp: 'lr 4/6/2009 23:58'!
cleanup
	"Execute all the registered cleanup code. Do not execute this code within an image you intend to continue using. This is a non-reversible operation and pretty much destroys your complete environment."

	(Pragma allNamed: #cleanup from: self class to: PRSegmentPersistency)
		do: [ :pragma | self perform: pragma selector ].
	Smalltalk garbageCollect! !

!PRSegmentPersistency methodsFor: 'cleanup' stamp: 'lr 7/9/2010 07:31'!
cleanupKom
	"Stop all the server adaptors, they just get into our way."
	<cleanup>
	
	WAServerManager shutDown! !

!PRSegmentPersistency methodsFor: 'cleanup' stamp: 'jf 9/15/2009 18:03'!
cleanupMagritte
	"Clear the magritte cache that shares common description objects with the kernel."
	<cleanup>
	
	MADescriptionBuilder default flush! !

!PRSegmentPersistency methodsFor: 'cleanup' stamp: 'lr 1/30/2009 08:21'!
cleanupMonticello
	"Clear monticello caches, they might reference some of our blocks in obscure ways."
	<cleanup>
	
	MCMethodDefinition shutDown.
	MCFileBasedRepository flushAllCaches! !

!PRSegmentPersistency methodsFor: 'cleanup' stamp: 'lr 1/30/2009 08:04'!
cleanupPier
	"Clear all registered kernels. We don't want references to our model from outside."
	<cleanup>
	
	PRKernel instVarNamed: 'Instances' put: nil! !

!PRSegmentPersistency methodsFor: 'cleanup' stamp: 'lr 10/13/2010 13:08'!
cleanupProcesses
	"Get rid of pocesses that potentialy still hang somewhere and reference our model. Take care not to kill any of the vital processes."
	<cleanup>

	| protected |
	protected := IdentitySet new.
	protected
		add: Project uiProcess;
		add: Processor activeProcess;
		add: Processor backgroundProcess;
		add: Smalltalk lowSpaceWatcherProcess;
		add: WeakArray runningFinalizationProcess.
	Process allInstances do: [ :each |
		(each isTerminated or: [ Processor highestPriority = each priority or: [ protected includes: each ] ])
			ifFalse: [ each terminate ] ]
	.
	! !

!PRSegmentPersistency methodsFor: 'cleanup' stamp: 'lr 10/13/2010 11:56'!
cleanupSeaside
	"Cleanup all the Seaside related objects such as dispatchers, sessions and configurations."
	<cleanup>
	
	WAAdmin clearAll! !

!PRSegmentPersistency methodsFor: 'cleanup' stamp: 'lr 1/30/2009 08:18'!
cleanupWorld
	"Close all windows, just in case we have some workspaces or pier browsers open that reference our model."
	<cleanup>

	World submorphs do: [ :morph | morph delete ].
	World releaseCachedState! !

!PRSegmentPersistency methodsFor: 'accessing' stamp: 'lr 2/5/2009 16:35'!
directory
	^ (FileDirectory default directoryNamed: 'snapshots') assureExistence! !

!PRSegmentPersistency methodsFor: 'accessing' stamp: 'lr 2/5/2009 16:30'!
namePrefix
	^ namePrefix! !

!PRSegmentPersistency methodsFor: 'accessing' stamp: 'lr 2/5/2009 16:30'!
namePrefix: aString
	namePrefix := aString! !

!PRSegmentPersistency methodsFor: 'private' stamp: 'lr 1/30/2009 08:34'!
serialize: aString
	| stream pointers |
	stream := self directory forceNewFileNamed: aString.
	pointers := [ [ ImageSegment new writeKernel: self kernel on: stream ]
		ensure: [ stream close ] ] on: Exception do: [ :error | nil ].
	(pointers isNil or: [ pointers anySatisfy: [ :each | (each isLiteral or: [ each isVariableBinding ]) not ] ]) 
		ifTrue: [ self directory deleteFileNamed: aString ifAbsent: [ ] ]! !

!PRSegmentPersistency methodsFor: 'actions' stamp: 'jf 11/3/2009 11:08'!
snapshot
	| os filename |
	(os := Smalltalk classNamed: #OSProcess) isNil
		ifTrue: [ ^ self message: 'OSProcess not installed.' ].
	os thisOSProcess pid
		ifTrue: [ ^ self message: 'OSProcess does not support this platform.' ].
	filename := self namePrefix , DateAndTime now asSortableString , '.obj'.
	os thisOSProcess forkHeadlessSqueakAndDo: [
		[ self cleanup; serialize: filename ]
			ensure: [ Smalltalk quitPrimitive ] ].
	self message: 'Last snapshot ' , filename printString , ' at ' , GRPlatform current magritteTimeStampClass now greaseString! !

!PRHistoryPersistency methodsFor: '*pier-pharo-persistency' stamp: 'lr 2/6/2009 22:31'!
sanitize
	super sanitize.
	history := OrderedCollection new! !

!PRKernel methodsFor: '*pier-pharo-persistency' stamp: 'lr 10/13/2010 12:27'!
exportOn: anExporter
	| persistencyBackup |
	anExporter sanitizeExport: self root.
	
	persistencyBackup := persistency.
	persistency := persistencyBackup class new
		copyFrom: persistencyBackup;
		sanitize.
			
	[ [ anExporter writeStream nextPut: self; close ] 
		ensure: [ persistency := persistencyBackup ] ]
			on: Error
			do: [ :err | ^ anExporter owner inform: 'Unable to export: ' , err messageText printString , '.' ].
! !

!PRPersistency methodsFor: '*pier-pharo-persistency' stamp: 'lr 2/6/2009 22:33'!
desanitize
	"Sanitizes the receiver to be serializeable."
	
	mutex := self defaultMutex! !

!PRPersistency methodsFor: '*pier-pharo-persistency' stamp: 'lr 2/6/2009 22:32'!
sanitize
	"Sanitizes the receiver to be serializeable."
	
	mutex := nil! !

!GRPackage class methodsFor: '*pier-pharo-persistency' stamp: 'YanniChiu 3/10/2010 22:55'!
pierPharoPersistency
	^ self new
		name: 'Pier-Pharo-Persistency';
		addDependency: 'Pier-Model';
		addDependency: 'Pier-Seaside';
		url: #pierUrl;
		yourself! !

PRWidget subclass: #PRExportImportWidget
	instanceVariableNames: 'exporterImporter replace'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Pharo-Persistency'!

!PRExportImportWidget class methodsFor: 'accessing' stamp: 'lr 2/5/2009 22:52'!
groupLabel
	^ 'Pier System'! !

!PRExportImportWidget class methodsFor: 'testing' stamp: 'lr 2/5/2009 22:44'!
isAbstract
	^ false! !

!PRExportImportWidget class methodsFor: 'accessing' stamp: 'lr 2/5/2009 22:44'!
label
	^ 'Export/Import'! !

!PRExportImportWidget methodsFor: 'actions' stamp: 'jmck 10/9/2010 16:48'!
export: aKernel
	self exporterImporter export: aKernel on: self requestContext! !

!PRExportImportWidget methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:14'!
exporterImporter
	^ exporterImporter ifNil: [ 
		self exporterImporter: PRReferenceStreamExporterImporter new.
		self exporterImporter ]! !

!PRExportImportWidget methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:17'!
exporterImporter: anExporterImporter
	exporterImporter := anExporterImporter owner: self! !

!PRExportImportWidget methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:13'!
exporterImporterClass
	^ self exporterImporter class! !

!PRExportImportWidget methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:13'!
exporterImporterClass: aClass
	self exporterImporter: aClass new! !

!PRExportImportWidget methodsFor: 'actions' stamp: 'jmck 10/9/2010 16:54'!
import: aCollection into: aKernel
	| newKernel |
	newKernel := self exporterImporter import: aCollection into: aKernel.
	self context: (self context structure: newKernel root)! !

!PRExportImportWidget methodsFor: 'initialization' stamp: 'lr 10/13/2010 12:22'!
initialize
	super initialize.
	self replace: false! !

!PRExportImportWidget methodsFor: 'rendering' stamp: 'lr 10/13/2010 12:16'!
renderContentOn: html
	html form 
		multipart; 
		with: [
			self renderSelectOn: html.
			self renderExportOn: html.
			self renderImportOn: html ]! !

!PRExportImportWidget methodsFor: 'rendering' stamp: 'lr 10/13/2010 12:16'!
renderExportOn: html
	html heading level: 2; with: 'Export'.
	html paragraph: 'Exporting a snapshot of the kernel does not include the history or any external files. If you plan to move the site to a new location make sure to copy the complete directory structure below ' , MAExternalFileModel baseDirectory pathName printString , ' separately. Depending on the size of your site it might take a while to generate and download the snapshot.'.
	html paragraph: [
		html submitButton
			callback: [ self export: self context kernel ];
			with: 'Export' ]! !

!PRExportImportWidget methodsFor: 'rendering' stamp: 'lr 10/13/2010 12:57'!
renderImportOn: html
	| contents |
	html heading level: 2; with: 'Import'.
	html paragraph: 'Importing a snapshot either replaces the existing kernel or just adds all included pages to the current kernel. In either case, make sure to merge the external files into ' , MAExternalFileModel baseDirectory pathName printString , ' and to check security properties.'.
	html paragraph: [
		html radioGroup: [ :group |
			html label: [
				html radioButton
					group: group;
					selected: self replace not;
					callback: [ self replace: false ].
				html text: 'Import Pages' ].
			html break.
			html label: [ 
				html radioButton
					group: group;
					selected: self replace;
					callback: [ self replace: true ].
				html text: 'Replace Kernel' ] ] ].
	html paragraph: [
		html fileUpload
			callback: [ :file | contents := file rawContents ] ].
	html paragraph: [
		html submitButton
			callback: [ self exporterImporter import: contents into: self context kernel ];
			with: 'Import' ]! !

!PRExportImportWidget methodsFor: 'rendering' stamp: 'lr 10/13/2010 12:17'!
renderSelectOn: html
	html paragraph: 'Please select the importer/exporter to use:'.
	html paragraph: [
		html select
			labels: [ :value | value label ];
			selected: self exporterImporterClass;
			list: PRExporterImporter withAllConcreteClasses;
			callback: [ :value | self exporterImporterClass: value ] ]! !

!PRExportImportWidget methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:21'!
replace
	^ replace! !

!PRExportImportWidget methodsFor: 'accessing' stamp: 'lr 10/13/2010 12:21'!
replace: aBoolean
	replace := aBoolean! !

!DateAndTime methodsFor: '*pier-pharo-persistency' stamp: 'lr 6/17/2010 09:39'!
asSortableString
	"A small interpreter to get a date exactly the way required."

	| code separator |
	code := #( year 4 $- monthIndex 2 $- dayOfMonth 2 $_ hour 2 $- minute 2 $- second 2 ) readStream.
	^ String streamContents: [ :stream |
		[ code atEnd ] whileFalse: [
			stream nextPutAll: ((self perform: code next)
				printPaddedWith: $0 to: code next).
			(separator := code next) isNil
				ifFalse: [ stream nextPut: separator ] ] ]! !
